/****
 *
 * Ttulo: Criando um Novo Comando de Sistema Operacional
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/07/2012
 * ltima modificao: 16/07/2012
 *
 * Descrio: Calcula a soma dos valores inteiros introduzidos
 *            como argumentos de linha de comando
 *
 * Entrada: Valores inteiros introduzidos via linha de comando
 *
 * Sada: O resultado da soma ou uma mensagem informando
 *        que no foi possvel calcul-la
 *
 ****/

#include <stdio.h>  /* printf()              */
#include <string.h> /* strchr()              */
#include <ctype.h>  /* isspace() e isdigit() */

/****
 *
 * StrEmInt(): Converte um string em nmero inteiro (int)
 *             na base 10
 *
 * Parmetros:
 *      str (entrada) - string a ser convertido
 *     *final (sada) - se no for NULL, apontar para
 *                      o caractere no qual a converso
 *                       encerrada
 *
 * Retorno: O nmero resultante da converso, se ela for
 *          bem sucedida, ou zero, se ocorrer erro
 *
 * Observaes:
 *     1. Se houver erro de converso, o valor retornado
 *         ambguo, mas a ambiguidade pode ser resolvida
 *        por meio do segundo parmetro.
 *
 *     2. Espaamento entre um sinal e o primeiro dgito
 *        do string  interpretado como erro.
 *
 ****/
int StrEmInt(const char *str, char **final)
{
   int digito, /* Armazenar cada dgito do nmero */
       ehNegativo = 0, /* Indicar se o nmero  negativo */
       numero = 0; /* Armazenar o resultado da converso */
   char *p, *inicio;

      /* Guarda o incio do string */
   inicio = (char *) str; /* Converso explcita para */
                          /* agradar o compilador     */

      /*                            */
      /* Salta eventuais espaos em */
      /* branco no incio do string */
      /*                            */
   while (isspace(*str)) {
      str++;
   }

      /* Verifica se o string possui sinal */
   if (*str == '-') {
      ehNegativo = 1; /* Indica que o nmero  negativo */
      ++str; /* Passa para o prximo caractere */
   } else if (*str == '+') { /* Sinal positivo no    */
                             /* levado em considerao */
      ++str; /* Passa para o prximo caractere */
   }

      /* Neste ponto foram saltados eventuais espaos */
      /* em branco iniciais e sinal. Agora, deve-se   */
      /* verificar se o string ficou vazio.           */
   if (!*str) {
         /* Se o ponteiro 'final' no for NULL, faz   */
         /* ele apontar para o incio do string.      */
      if (final) {
         *final = inicio;
      }

      return 0; /* No houve converso alguma */
   }

      /*                                                      */
      /* Salta eventuais espaos em branco ao final do string */
      /*                                                      */

      /* Faz p apontar para o ltimo caractere do string */
   p = strchr(str, '\0') - 1;

      /* Salta espaos em branco ao final do string */
   while (isspace(*p)) {
      p--;
   }

      /* Neste ponto, p aponta para o ltimo caractere do */
      /* string que no  espao em branco. Faz-se este   */
      /* ponteiro apontar para o caractere seguinte para  */
      /* que ele se torne delimitador dos caracteres do   */
      /* string que sero convertidos.                    */
   ++p;

      /* O lao encerra quando o string delimitado */
      /* por p for totalmente explorado ou se      */
      /* ocorrer algum erro durante a converso    */
   while (str != p) {
      if (isdigit(*str)) { /* Mais um dgito encontrado */
            /* Obtm o nmero equivalente ao dgito */
         digito = *str - '0'; /* Vlido para qualquer */
                              /* cdigo de caracteres */

            /* Atualiza o valor numrico (10  a base) */
         numero = numero*10 + digito;
      } else { /* Caractere esprio encontrado */
            /* Se o parmetro 'final' no for NULL, faz */
            /* o ponteiro apontar para o caractere que  */
            /* causou o final da converso              */
         if (final) {
            *final = (char *) str; /* Se (char *) no for */
                        /* usado o compilador apresentar */
                        /* mensagem de advertncia        */
         }

            /* O resultado obtido at aqui ser retornado,*/
            /* mas a converso pode ter sido mal sucedida */
         return numero;
      }

         /* O nmero sendo convertido  positivo. Assim, se  */
         /* ele assumir um valor negativo, ocorreu overflow. */
      if (numero < 0) {
         printf("\a\n\t>>> Overflow!!! <<<\n");

            /* Se o parmetro 'final' no for NULL, faz */
            /* o ponteiro apontar para o caractere que  */
            /* causou o final da converso              */
         if (final) {
            *final = (char *) str; /* (char *) agrada */
                                   /* o compilador    */
         }

         return 0; /* A converso foi mal sucedida */
      }

      str++; /* Passa para o prximo caractere */
   }

      /* Se o nmero for negativo, efetua a correo */
   if (ehNegativo) {
      numero = -numero; /* O nmero  negativo */
   }

      /* Se o parmetro 'final' no for NULL, faz o */
      /* ponteiro apontar para o caractere terminal */
   if (final) {
      *final = strchr(str, '\0');
   }

   return numero; /* A converso foi bem sucedida */
}

/****
 *
 * main(): Calcula a soma dos valores inteiros introduzidos como
 *         argumentos de linha de comando
 *
 * Parmetros:
 *     argc (entrada) - Nmero de argumentos de linha de comando
 *     argv (entrada) - Array de strings presentes na linha de
 *                      comando quando o programa o programa 
 *                      executado
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 * Observao: Se algum valor introduzido em linha de comando no
 *             puder ser convertido, a soma no ser calculada.
 *
 ****/
int main(int argc, char *argv[])
{
   int   i, valor, soma = 0;
   char *p;

      /* Deve haver pelo menos trs argumentos */
      /* de linha de comando                   */
   if (argc < 3) {
      printf( "\n\t>>> Este programa deve ser usado assim:"
              "\n\t>>> %s n1 n2 ..., sendo n1, n2, ... "
              "inteiros\n",  argv[0]);
      return 1;
   }

      /* Converte e soma as parcelas representadas */
      /* por argumentos de linha de comando        */
   for (i = 1; i < argc; ++i) {
         /* Tenta converter um argumento */
         /* em inteiro na base decimal   */
      valor = StrEmInt(argv[i], &p);

         /* Se p aponta para '\0', a converso foi OK. */
         /* Caso contrrio, a converso no foi        */
         /* possvel. Ento, encerra-se o programa.    */
      if (*p) {
         printf( "\n\t>>> O valor \"%s\" nao pode ser "
                 "convertido em inteiro\n", argv[i] );
         return 1;
      }

         /* Acrescenta o argumento convertido  soma */
      soma = soma + valor;
   }

   printf("\n\t>>> Soma: %d\n", soma);

   return 0;
}

/******************* Exemplos de Execuo ******************

Exemplo 1:
==========

C:\>somaints

        >>> Este programa deve ser usado assim:
        >>> NovoComando n1 n2 ..., onde n1, n2, ... sao inteiros

Exemplo 2:
==========

C:\>somaints -12 32 55

        >>> Soma: 75

Exemplo 3:
==========

C:\>somaints -12 32 55ab

        >>> O valor 55ab nao pode ser convertido em inteiro

***********************************************************/
