/****
 *
 * Ttulo: Calculando o Tamanho de um Arquivo
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2013
 *
 * Descrio: Determina o tamanho de um arquivo em bytes
 *
 * Entrada: Nome do arquivo introduzido via linha de comando
 *
 * Sada: Uma mensagem informando o nmero de bytes do
 *        referido arquivo
 *
 ****/

#include <stdio.h>  /* Entrada e sada */
#include <stdlib.h> /* Funo exit()   */

/****
 *
 * TamanhoDeArquivo(): Calcula o tamanho de um arquivo lendo cada
 *                     byte que o constitui e contando quantos
 *                     bytes so lidos
 *
 * Parmetros:
 *     stream (entrada) - stream binrio associado ao arquivo cujo
 *                        tamanho ser calculado
 *
 * Retorno: -1, se ocorrer algum erro.
 *          O tamanho do arquivo, em caso contrrio.
 *
 * Observaes:
 *     1. O arquivo deve estar aberto em modo binrio que permite
 *        leitura
 *     2. A posio atual do indicador de posio do arquivo 
 *        preservada
 *     3. O arquivo deve permitir acesso direto
 *
 ****/
int TamanhoDeArquivo(FILE *stream)
{
   int posicaoAtual, tamanho = 0;

      /* Guarda a posio atual do indicador de posio */
      /* do arquivo para que ele possa ser restaurado   */
   posicaoAtual = ftell(stream);

      /* Verifica se ftell() foi bem sucedida */
   if (posicaoAtual < 0) {
      return -1; /* A chamada de ftell() falhou */
   }

      /* Assegura leitura a partir do incio do arquivo */
   if ( fseek(stream, 0, SEEK_SET) ) {
         /* O movimento do indicador de posio no foi   */
         /* possvel. Nesse caso, no precisa restaur-lo */
      return -1;
   }

      /* O lao encerra quando o final do arquivo for */
      /* atingido ou se ocorrer erro de leitura       */
   while (1){
      fgetc(stream); /* L e descarta o prximo byte */

         /* Checa se o arquivo j foi inteiramente lido */
      if (feof(stream)) {
         break; /* Encerra lao */
      }

         /* Verifica se ocorreu erro de leitura */
      if (ferror(stream)) {
            /* Restaura a posio original do indicador */
            /* de posio do arquivo  antes de retornar */
         fseek(stream, posicaoAtual, SEEK_SET);
         return -1; /* Ocorreu erro de leitura */
      }

      ++tamanho; /* Mais um byte foi lido */
   }

      /* Restaura a posio original do indicador de posio */
   if ( fseek(stream, posicaoAtual, SEEK_SET) ) {
      return -1; /* O movimento no foi possvel */
   }

   return tamanho;
}

/****
 *
 * FechaArquivo(): Tenta fechar um arquivo e, quando isso
 *                 no  possvel, aborta o programa
 *
 * Parmetros:
 *      stream (entrada) - stream associado ao arquivo
 *      nomeArq (entrada) - nome do arquivo ou NULL
 *
 * Retorno: Nada
 *
 * Observao: Se o segundo parmetro no for NULL, o nome
 *             do programa aparece na mensagem de erro
 *
 ****/
void FechaArquivo(FILE *stream, const char *nomeArq)
{
   /***************************************************/
   /*                                                 */
   /* Se fclose() retornar um valor diferente de zero */
   /* ocorreu algum erro na tentativa de fechamento   */
   /* do arquivo. Neste caso, apresenta-se mensagem   */
   /* de erro e aborta-se o programa.                 */
   /*                                                 */
   /***************************************************/

   if (fclose(stream)) { /* Erro de fechamento */
      fprintf( stderr, "\a\n>>> Ocorreu erro no fechamento"
              " do arquivo %s.\n>>> O programa sera' "
              "encerrado.\n", nomeArq ? nomeArq : "");
      exit(1); /* Aborta o programa */
   }
}

/****
 *
 * main(): Determina o tamanho em bytes do arquivo cujo nome 
 *         introduzido via linha de comando
 *
 * Parmetros:
 *     argc (entrada) - Nmero de argumentos de linha de comando
 *     argv (entrada) - Array de strings presentes na linha de
 *                      comando quando o programa  executado
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main(int argc, char *argv[])
{
   FILE  *stream;
   int    tamArquivo;

      /* Verifica se o programa foi invocado corretamente */
   if (argc != 2) { /* No foi */
      printf( "\n\t>>> Este programa deve ser usado assim:"
              "\n\t>>> %s nome-do-arquivo\n", argv[0] );
      return 1; /* Usurio no sabe usar o programa */
   }

      /* Tenta abrir o arquivo para leitura em modo */
      /* binrio e checa se o arquivo foi aberto    */
   if (!(stream = fopen(argv[1], "rb"))) {
      printf("\n\t> Impossivel abrir o arquivo %s\n", argv[1]);
      return 1; /* Erro de abertura de arquivo */
   }

      /* Calcula o tamanho do arquivo */
   tamArquivo = TamanhoDeArquivo(stream);

      /* Apresenta o resultado */
   if (tamArquivo < 0) {
      printf( "\n\t> Nao foi possivel determinar o tamanho "
              "de \"%s\" em modo binario", argv[1] );
   } else {
      printf( "\n\t> Tamanho de \"%s\" em modo binario: "
              "%d bytes\n", argv[1], tamArquivo );
   }

   FechaArquivo(stream, argv[1]); /* Fecha o arquivo */

      /* Tenta abrir o arquivo para leitura em modo */
      /* de texto e checa se o arquivo foi aberto   */
   if (!(stream = fopen(argv[1], "r"))) {
      printf("\n\t> Impossivel abrir o arquivo %s\n", argv[1]);
      return 1; /* Erro de abertura de arquivo */
   }

      /* Calcula o tamanho do arquivo */
   tamArquivo = TamanhoDeArquivo(stream);

      /* Apresenta o resultado */
   if (tamArquivo < 0) {
      printf( "\t> Nao foi possivel determinar o tamanho de"
              " \"%s\" em modo de texto", argv[1] );
   } else {
      printf( "\t> Tamanho de \"%s\" em modo de texto: "
              "%d bytes\n", argv[1], tamArquivo );
   }

      /* Aqui, no  preciso usar FechaArquivo(),  */
      /* pois o programa ser encerrado em seguida */
   fclose(stream);

   return 0;
}

/******************* Exemplos de Execuo ******************

C:\Programas> tamanho

        >>> Este programa deve ser usado assim:
        >>> tamanho nome-do-arquivo

C:\Programas> tamanho Tudor.txt

        > Tamanho de "Tudor.txt" em modo binario: 197 bytes
        > Tamanho de "Tudor.txt" em modo de texto: 190 bytes

C:\Programas> tamanho Tudor.bin

        > Tamanho de "Tudor.bin" em modo binario: 336 bytes
        > Tamanho de "Tudor.bin" em modo de texto: 94 bytes

[O nome do programa executvel  tamanho.exe]

***********************************************************/
