/****
 *
 * Ttulo: Jogo de Palavras (com Ajuda dos Universitrios)
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: L uma palavra de um arquivo contendo uma palavra por
 *            linha, embaralha a palavra, apresenta a palavra
 *            embaralhada para o usurio e desafia-o a adivinhar
 *            qual  a palavra.
 *
 * Entrada: A palavra lida no arquivo.
 *
 * Sada: Uma mensagem informando o usurio se ele acertou qual 
 *        a palavra ou no.
 *
 * Observao: A lista de palavras utilizado por este programa foi
 *             criada e gentilmente   Valdir Jorge, que  analista
 *             de sistemas e programador na Universidade Concrdia,
 *             em Montreal, Canad.
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Entrada e sada             */
#include <stdlib.h>  /* Alocao dinmica           */
#include <time.h>    /* Funo time()               */
#include <string.h>  /* Processamento de strings    */
#include <ctype.h>   /* Classificao de caracteres */
#include "leitura.h" /* LeituraFacil                */

/***************** Constantes Simblicas ******************/

   /* Nome do arquivo que contm as palavras */
#define ARQ_PALAVRAS "ListaDePalavras.txt"

#define MAX_PALAVRA  50 /* Tamanho mximo de uma palavra */
#define CHANCES      5  /* Nmero mximo de chances que  */
                        /* o usurio ter para adivinhar */
#define TAMANHO_BLOCO 256 /* Tamanho do acrscimo do   */
                          /* bloco usado na leitura a  */
                          /* cada chamada de realloc() */

/************************* Aluses ************************/

extern void FechaArquivo(FILE *stream, const char *nomeArq);
extern int NumeroDeLinhas(FILE*  stream);
extern char *LeLinhaIlimitada(int *tam, FILE *stream);
extern char* ConverteEmMaiusculas(char *str);
extern char *EmbaralhaPalavra( char *embaralho,
                               const char *palavra );
extern int AjudaUniversitarios(const char *palavra, int n);

/***************** Definies de Funes ******************/

/****
 *
 * FechaArquivo(): Tenta fechar um arquivo e, quando isso
 *                 no  possvel, aborta o programa
 *
 * Parmetros:
 *      stream (entrada) - stream associado ao arquivo
 *      nomeArq (entrada) - nome do arquivo ou NULL
 *
 * Retorno: Nada
 *
 * Observao: Se o segundo parmetro no for NULL, o nome
 *             do programa aparece na mensagem de erro
 *
 ****/
void FechaArquivo(FILE *stream, const char *nomeArq)
{
   /***************************************************/
   /*                                                 */
   /* Se fclose() retornar um valor diferente de zero */
   /* ocorreu algum erro na tentativa de fechamento   */
   /* do arquivo. Neste caso, apresenta-se mensagem   */
   /* de erro e aborta-se o programa.                 */
   /*                                                 */
   /***************************************************/

   if (fclose(stream)) { /* Erro de fechamento */
      fprintf( stderr, "\a\n>>> Ocorreu erro no fechamento"
              " do arquivo %s.\n>>> O programa sera' "
              "encerrado.\n", nomeArq ? nomeArq : "");
      exit(1); /* Aborta o programa */
   }
}

/****
 *
 * NumeroDeLinhas(): Conta o nmero de linhas de um
 *                   arquivo de texto
 *
 * Parmetros:
 *      stream (entrada): stream associado ao arquivo
 *
 * Retorno: O nmero de linhas do arquivo ou
 *          um valor negativo se ocorrer erro
 *
 * Observao: O arquivo deve ter sido aberto em modo de
 *             texto que permite leitura
 *
 ****/
int NumeroDeLinhas(FILE  *stream)
{
   int    c, /* Armazena temporariamente um caractere */
          linhas = 0; /* Armazena o total de linhas */

      /* Garante leitura a partir do incio do arquivo */
   rewind(stream);

      /* L cada caractere do stream e conta quantos   */
      /* caracteres '\n' so encontrados. O lao while */
      /* encerra quando o final do arquivo  atingido  */
      /* ou ocorre algum erro de leitura.              */
   while (1) {
      c = fgetc(stream); /* L um caractere no stream */

         /* Se o final do arquivo foi atingido    */
         /* ou ocorreu algum erro, encerra o lao */
      if ( feof(stream) || ferror(stream) )
         break;

      if (c == '\n') {
         linhas++; /* Mais uma linha encontrada */
      }
   }

      /* Se ocorreu erro retorna -1. Caso      */
      /* contrrio, retorna o nmero de linhas */
   return ferror(stream) ? -1 : linhas;
}

/****
 *
 * LeLinhaIlimitada(): L uma linha de tamanho arbitrrio
 *                     num stream de texto e armazena os
 *                     caracteres lidos num array alocado
 *                     dinamicamente
 *
 * Parmetros:
 *     *tam (sada) - se no for NULL, armazenar o tamanho
 *                    do string constitudo pelos caracteres
 *                    da linha
 *      stream (entrada) - stream de texto no qual ser
 *                         feita a leitura
 *
 * Retorno: Endereo do array contendo a linha lida.
 *          NULL, se ocorrer erro ou o final do arquivo
 *          for atingido antes da leitura de qualquer
 *          caractere
 *
 * Observaes:
 *
 *       1. O stream deve estar associado a um arquivo de
 *          texto aberto em modo de texto que permita
 *          leitura
 *
 *       2. O caractere '\n' no  includo no string
 *          resultante da leitura
 *
 *       3. O primeiro parmetro pode ser NULL. Neste
 *          caso, o tamanho do string no ser armazenado
 *
 ****/
char *LeLinhaIlimitada(int *tam, FILE *stream)
{
   char *ar = NULL,   /* Ponteiro para um array      */
                      /* alocado dinamicamente que   */
                      /* conter os caracteres lidos */
        *p; /* Usado em chamada de realloc() */
   int   tamanho = 0, /* Tamanho do array alocado */
         c,  /* Armazenar cada caractere lido */
         i; /* ndice do prximo caractere */
            /* a ser inserido no array     */

      /* L caracteres a partir da posio corrente do */
      /* apontador de posio do arquivo e armazena-os */
      /* num array. A leitura encerra quando '\n'     */
      /* encontrado, o final do arquivo  atingido ou  */
      /* ocorre erro.                                  */
   for (i = 0; ; ++i) {
         /* L o prximo caractere no arquivo */
      c = fgetc(stream);

         /* Se ocorreu erro de leitura, libera o  */
         /* bloco eventualmente alocado e retorna */
      if (ferror(stream)) {
         free(ar); /* Libera o bloco apontado por 'ar' */
         return NULL; /* Ocorreu erro de leitura */
      }

         /* Verifica se array est completo.      */
         /* Se for o caso, tenta redimension-lo. */
      if (i > tamanho - 2) {
         p = realloc(ar, tamanho + TAMANHO_BLOCO);

            /* Se o redimensionamento no foi     */
            /* possvel, libera o bloco e retorna */
         if (!p) {
            free(ar); /* Libera o bloco apontado por 'ar' */
            return NULL; /* Ocorreu erro de alocao */
         }

            /* Redimensionamento foi OK. Ento, faz-se */
            /* 'ar' apontar para o novo bloco.         */
         ar = p;

            /* O array aumentou de tamanho */
         tamanho = tamanho + TAMANHO_BLOCO;
      }

         /* Se final do arquivo for atingido */
         /* ou o caractere '\n' for lido,    */
         /* encerra a leitura                */
      if (feof(stream) || c == '\n') {
         break; /* Leitura encerrada */
      }

         /* Acrescenta mais um caractere ao array */
      ar[i] = c;
   }

      /* Se nenhum caractere foi lido, libera */
      /* o espao alocado e retorna NULL      */
   if (feof(stream) && !i) {
      free(ar); /* Libera o bloco apontado por 'ar' */
      return NULL; /* Nenhum caractere foi */
                   /* armazenado no array  */
   }

      /* Insere o caractere terminal de string. Neste  */
      /* instante, deve haver espao para ele porque o */
      /* array foi sempre redimensionado deixando um   */
      /* espao para o onipresente caractere '\0'.     */
   ar[i] = '\0';

      /* Atualiza valor apontando pelo parmetro */
      /* 'tam' se ele no for NULL               */
   if (tam) {
      *tam = i; /* i contm o tamanho do array */
   }

   /*                                     */
   /* >>> NB: O tamanho do string no <<< */
   /* >>>     inclui o caractere '\0' <<< */
   /*                                     */

      /* Tenta ajustar o tamanho do array para  */
      /* no haver desperdcio de memria. Note */
      /* que, como i  o tamanho do string, o   */
      /* tamanho do array deve ser i + 1.       */
   p = realloc(ar, i + 1);

   /*                                                 */
   /* Se a realocao foi bem sucedida, retorna-se p. */
   /* Caso contrrio, ar ainda aponta para um bloco   */
   /* vlido. Talvez, haja desperdcio de memria,    */
   /* mas, aqui,  melhor retornar ar do que NULL.    */
   /*                                                 */

   return p ? p : ar;
}

/****
 *
 * ConverteEmMaiusculas(): Converte as letras de um string em
 *                         maisculas. Caracteres que no so
 *                         letras no so afetados.
 *
 * Parmetros: str (entrada/sada): o string
 *
 * Retorno: O endereo do string recebido como parmetro
 *
 ****/
char* ConverteEmMaiusculas(char *str)
{
   char *p = str; /* Guarda incio do string */

      /*                                     */
      /* >>> Converte em maiscula cada <<<  */
      /* >>> caractere que  letra.     <<<  */
      /*                                     */
   while (*str) {
      *str = toupper(*str); /* Converte caractere atual */
      ++str; /* Passa para o prximo caractere */
   }

   return p; /* Retorna o endereo do string */
}

/****
 *
 * EmbaralhaPalavra(): Embaralha os caracteres de um string
 *
 * Parmetros:
 *      embaralho (sada) - o string embaralhado
 *      palavra (entrada) - string que ser embaralhado
 *
 * Retorno: Endereo do string embaralhado, se no ocorrer erro.
 *          NULL, se ocorrer erro.
 *
 * Observao: Esta funo no verifica se o string contm apenas
 *             letras. Portanto, ela serve para embaralhar
 *             qualquer tipo de string.
 *
 ****/
char *EmbaralhaPalavra(char *embaralho, const char *palavra)
{
   int   i, j, tam;
   char *p;

   tam = strlen(palavra); /* Calcula o tamanho da palavra */

      /* Aloca um array auxiliar. Acrescenta-se 1  */
      /* ao tamanho da palavra para levar em conta */
      /* o caractere terminal de string            */
   p = malloc(tam + 1);

      /* Se o array no foi alocado, no  possvel continuar */
   if (!p) {
      return NULL; /* Game over! */
   }

      /* Copia o string que ser embaralhado */
      /* para o array auxiliar               */
   strcpy(p, palavra);

      /* Embaralha o string trocando os caracteres de posio */
   for (i = 0; i < tam; ++i) {
      while (1) {
            /* Sorteia o caractere do string apontado */
            /* por p que ser armazenado na posio i */
            /* do array apontado por 'embaralho'      */
         j = rand()%tam;

         /****************************************************/
         /* Caracteres que j foram usados so substitudos  */
         /* por '\0' no array p[]. A instruo if verifica   */
         /* se o caractere sorteado j foi usado. Se esse    */
         /* no for o caso, armazena-se o caractere no array */
         /* embaralho[], substitui-se o caractere usado no   */
         /* array p[] por '\0' e passa-se para o prximo     */
         /* caractere do array embaralho[]. Se o caractere   */
         /* j tiver sido usado, volta-se ao incio do lao  */
         /* while e faz-se novo sorteio.                     */
         /****************************************************/

         if (p[j] != '\0') {
              /* O caractere de ndice j do string apontado por */
              /* p ainda no foi usado no embaralho. Ento, ele */
              /*  armazenado no array que contm  o embaralho  */
            embaralho[i] = p[j];

               /* Agora, o caractere de ndice j j foi usado   */
               /* no  embaralho. Assim, coloca-se um caractere  */
               /* terminal nessa posio para indicar esse fato.*/
            p[j] = '\0';

            break; /* Encerra o lao while */
         }
      } /* while */
   } /* for */

      /* Armazena o caractere terminal no array      */
      /* embaralho[] para que ele contenha um string */
   embaralho[tam] = '\0';

      /* Libera espao ocupado pelo array auxiliar, */
      /* pois ele no  mais necessrio             */
   free(p);

   return embaralho;
}

/****
 *
 * AjudaUniversitarios(): Oferece ajuda ao jogador
 *
 * Parmetros: palavra (entrada) - a palavra para a qual ser
 *                                 apresentada ajuda
 *             n (entrada) - ndice da ajuda
 *
 * Retorno: 1, quando  concedida ajuda. 0, em caso contrrio.
 *
 ****/
int AjudaUniversitarios(const char *palavra, int n)
{
      /* Seleciona o nvel de ajuda */
   switch (n) {
      case 1: /* Primeiro nvel de ajuda */
         printf("\n\t>>> A primeira letra e': %c\n", *palavra);

         return 1; /* Uma ajuda foi concedida */

      case 2: /* Segundo nvel de ajuda */
         printf( "\n\t>>> A ultima letra e': %c\n",
                 *(strchr(palavra, '\0') - 1) );

         return 1; /* Outra ajuda foi concedida */

      case 3: /* Terceiro nvel de ajuda */
         printf( "\n\t>>> (Ultima ajuda) A segunda "
                 "letra e': %c\n", palavra[1] );

         return 1; /* Mais outra ajuda foi concedida */

      default: /* Nem em Silvio Santos h mais ajuda */
         printf("\n\t>>> Nao ha mais ajuda dos universitarios\n");

         return 0; /* No houve ajuda */
   }
}

/****
 *
 * main(): L uma palavra de um arquivo contendo uma palavra por
 *         linha, embaralha a palavra, apresenta a palavra
 *         embaralhada  ao usurio e desafia-o a adivinhar qual
 *          a palavra.
 *
 * Parmetros: Nenhum
 *
 * Retorno: 0, se no ocorrer nenhum erro; 1, em caso contrrio
 *
 ****/
int main(void)
{
   int   i,
         nPalavras, /* Nmero de palavras no arquivo */
         indicePalavra, /* ndice da palavra sorteada */
         tamanho, /* Tamanho da palavra sorteada no arquivo */
         nAjuda = 0; /* Nmero de auxlios    */
                     /* oferecidos ao usurio */
   char *palavra, /* Aponta para o array contendo */
                  /* a palavra sorteada           */
        *embaralho, /* Aponta para o array contendo */
                    /* a palavra embaralhada        */
        *chute; /* Ponteiro para o array que conter */
                /* cada tentativa do usurio         */
   FILE *stream; /* Stream associado ao arquivo que */
                 /* contm o banco de palavras      */

      /* Apresenta o programa */
   printf("\n\t>>> Este programa embaralha uma palavra e da' "
          "chances\n\t>>> para voce adivinhar qual e' a palavra "
          "original.\n\t>>> Digite '?' para solicitar ajuda aos "
          "universitarios\n\t>>> (3 vezes apenas), mas "
          "voce perde um chute a cada ajuda\n\n");

      /* Tenta abrir o arquivo contendo as   */
      /* palavras para leitura em modo texto */
   stream = fopen(ARQ_PALAVRAS, "r");

      /* Se o arquivo no foi aberto, */
      /* o jogo no pode continuar    */
   if (!stream) {
      printf( "\nNao foi possivel abrir o arquivo %s\n"
              "e o jogo sera' adiado\n", ARQ_PALAVRAS );
      return 1; /* Game over! */
   }

      /* Determina o nmero de palavras do arquivo, */
      /* que contm uma palavra por linha           */
   nPalavras = NumeroDeLinhas(stream);

   srand(time(NULL)); /* Inicia o gerador de nmeros aleatrios */

      /* Sorteia o ndice da palavra no    */
      /* arquivo. A indexao comea em 1. */
   indicePalavra = rand()%nPalavras + 1;

      /* Assegura que a leitura comea */
      /* no incio do arquivo          */
   rewind(stream);

      /* L e descarta cada palavra at */
      /* chegar  palavra sorteada      */
   for (i = 1; i < indicePalavra; ++i) {
         /* A funo LeLinhaIlimitada() aloca espao      */
         /* dinamicamente. Portanto,  necessrio liberar */
         /* o espao ocupado por cada linha lida.         */
      free(LeLinhaIlimitada(NULL, stream));
   }

      /* L no arquivo a palavra sorteada e armazena seu */
      /* nmero de caracteres na varivel 'tamanho'      */
   palavra = LeLinhaIlimitada(&tamanho, stream);

      /* O arquivo no  mais necessrio */
   FechaArquivo(stream, ARQ_PALAVRAS);

      /* Aloca espao para conter a palavra embaralhada. */
      /*  necessrio acrescentar 1 porque o tamanho da  */
      /* palavra no inclui o caractere terminal '\0'.   */
   embaralho = malloc(tamanho + 1);

      /* Tenta embaralhar a palavra. Se no */
      /* for possvel, encerra o programa.  */
   if ( !EmbaralhaPalavra(embaralho, palavra) ) {
      printf("\nOcorreu um erro no embaralho\n");
      return 1;
   }

      /* Desafia usurio apresentando-lhe a palavra embaralhada */
   printf( "\nVoce tem %d chances para adivinhar que "
           "palavra e' esta: %s\n", CHANCES, embaralho );

      /* Inicia 'chute' com NULL para no haver problema */
      /* na primeira chamada de free() no corpo do lao  */
   chute = NULL;

      /* L cada chute do usurio e verifica se */
      /* ele acertou qual  a palavra certa     */
   for (i = 1; i <= CHANCES; ++i) {
         /* Libera o espao ocupado pelo ltimo chute do        */
         /* usurio. No haver problema se ainda no houve     */
         /* chute pois a varivel 'chute' foi iniciada com NULL */
      free(chute);

         /* L um chute do usurio, testando */
         /* se ocorre erro de leitura        */
      printf("\n\t>>> %do. chute: ", i);
      if (!(chute = LeLinhaIlimitada(NULL, stdin))) {
         printf("\nOcorreu um erro de leitura\n");
         return 1; /* Se a funo LeLinhaIlimitada()      */
                /* retornou NULL, deve ter ocorrido erro. */
                /* Talvez o usurio tenha digitado        */
                /* ^D (Unix) ou ^Z (Windows/DOS).         */
      }

         /* Se o primeiro caractere do chute do usurio */
         /* for '?', interpreta-se que ele deseja ajuda */
      if (*chute == '?') {
         if (!AjudaUniversitarios(palavra, ++nAjuda)) {
               /* Se ocorreu ajuda, o nmero */
               /* de chutes  decrementado   */
            --i;
         }

            /* Salta o resto do lao pois no houve chute */
         continue;
      }

         /* Verifica se o chute do usurio foi      */
         /* correto. O chute precisa ser convertido */
         /* em letras maisculas porque as palavras */
         /* do arquivo so assim.                   */
      if (!strcmp(ConverteEmMaiusculas(chute), palavra)) {
         printf("\a\n\t>>> Parabens. Voce acertou. <<<\n");
         return 0; /* Se o usurio acertou, o jogo acaba */
      } else if (i < CHANCES) {
         printf("\nErrou. Tente novamente.\n");
      }
   }

      /* Se no houve retorno no corpo do ltimo */
      /* lao for, o usurio no acertou         */
   printf( "\n\t>>> Infelizmente, voce nao acertou."
           "\n\t>>> A palavra era: \"%s\"\n", palavra );

      /* Libera os espaos ocupados pela palavra */
      /* e pelo embaralho. Isso no  realmente  */
      /* necessrio, j que o programa encerrar */
      /* em seguida. Mas, preserva o bom hbito. */
   free(palavra);
   free(embaralho);

   return 0;
}

/******************* Exemplos de Execuo ******************

Execuo 1:
==========

        >>> Este programa embaralha uma palavra e da' chances
        >>> para voce adivinhar qual e' a palavra original.
        >>> Digite '?' para solicitar ajuda aos universitarios
        >>> (3 vezes apenas), mas voce perde um chute a cada ajuda

Voce tem 5 chances para adivinhar que palavra e' esta: EDEMDS

        >>> 1o. chute: ?

        >>> A primeira letra e': D

        >>> 2o. chute: ?

        >>> A ultima letra e': M

        >>> 3o. chute: ?

        >>> (Ultima ajuda) A segunda letra e': E

        >>> 4o. chute: desdem

        >>> Parabens. Voce acertou. <<<

Execuo 2:
==========

        >>> Este programa embaralha uma palavra e da' chances
        >>> para voce adivinhar qual e' a palavra original.
        >>> Digite '?' para solicitar ajuda aos universitarios
        >>> (3 vezes apenas), mas voce perde um chute a cada ajuda

Voce tem 5 chances para adivinhar que palavra e' esta: IEODDTNRSU

        >>> 1o. chute: ?

        >>> A primeira letra e': D

        >>> 2o. chute: ?

        >>> A ultima letra e': O

        >>> 3o. chute: ?

        >>> (Ultima ajuda) A segunda letra e': E

        >>> 4o. chute: ?

        >>> Nao ha' mais ajuda dos universitarios

        >>> 4o. chute: desmentido

Errou. Tente novamente.

        >>> 5o. chute: ?

        >>> Nao ha' mais ajuda dos universitarios

        >>> 5o. chute: desisto

        >>> Infelizmente, voce nao acertou.
        >>> A palavra era: "DESNUTRIDO"

***********************************************************/
